import numpy as np
import stan
import pickle

xy = [[25, 40], [57, 65], [60, 68], [38, 50], [55, 65],
      [45, 49], [36, 55], [42, 54], [54, 61], [44, 59],
      [33, 73], [47, 56], [39, 54], [44, 61], [58, 63],
      [53, 58], [32, 53], [52, 70], [55, 64], [21, 43]]
xy = np.array(xy)

with open('xy.pkl', 'wb') as f:
    pickle.dump(xy, f)

StanCode = """
    data {
        int n;
        array[n] vector[2] xy;
        real xmin;
        real xmax;
        real ymin;
        real ymax;
    }
    parameters {
        /*
                再スケール前のサンプル値
        */
        real<lower=0, upper=1> vrho;
        real<lower=0, upper=1> vxmu;
        real<lower=0, upper=1> vymu;
        real<lower=0, upper=1> vxsgm;
        real<lower=0, upper=1> vysgm;
    }
    transformed parameters {
        /*
             データに合わせてパラメータのスケーリングを行う
        */
        real rho;
        real xmu;
        real ymu;
        real xsgm;
        real ysgm;
        rho = -0.9999 + vrho*1.9998;
        xmu = xmin + (xmax - xmin) * vxmu;
        ymu = ymin + (ymax - ymin) * vymu;
        xsgm = 0.0001 + (xmax - xmin) * vxsgm;
        ysgm = 0.0001 + (ymax - ymin) * vysgm;
    }
    model {
        /*
                スケーリング前のパラメータの事前分布
        */
        vrho ~ uniform(0,1); 
        vxmu ~ uniform(0,1);  
        vymu ~ uniform(0,1); 
        vxsgm ~ uniform(0,1); 
        vysgm ~ uniform(0,1);
        /*
                 ２変量正規分布
        */
        xy ~ multi_normal([xmu, ymu],
                              [[xsgm^2, xsgm*ysgm*rho],
                              [ysgm*xsgm*rho, ysgm^2]]);
    }
"""

sm = stan.build(StanCode,
                data={'n':len(xy), 'xy':xy,
                      'xmin':np.min(xy.T[0]), 'xmax':np.max(xy.T[1]),
                      'ymin':np.min(xy.T[1]), 'ymax':np.max(xy.T[1])})
fit = sm.sample()

df_fit = fit.to_frame()     #  MCMCサンプルのDataFrame型への変換

xmus = df_fit['xmu']
xsgms = df_fit['xsgm']
ymus = df_fit['ymu']
ysgms = df_fit['ysgm']
rhos = df_fit['rho']
print(xmus.describe())
print(xsgms.describe())
print(rhos.describe())
"""
       MCMCサンプルの保存
"""
with open('rhos.pkl', 'wb') as f:
    pickle.dump(rhos, f)
with open('xmus.pkl', 'wb') as f:
    pickle.dump(xmus, f)
with open('xsgms.pkl', 'wb') as f:
    pickle.dump(xsgms, f)
with open('ymus.pkl', 'wb') as f:
    pickle.dump(ymus, f)
with open('ysgms.pkl', 'wb') as f:
    pickle.dump(ysgms, f)
