import stan       #  pystan3がimportされる
import pickle     #/  データの保存用モジュール

StanCode = """
    data {
        int N;    //  2項分布の総試行数
        int k;    //  生起回数
    }
    parameters {
        real<lower=0,upper=1> p;    //  生起確率
    }
    model {
        k ~ binomial(N, p);         //  2項分布のモデル
    }
"""

sm = stan.build(StanCode, data={'N':10, 'k':7})
fit = sm.sample()

df_fit = fit.to_frame()               #  PandasのDataFrameに変換
print(df_fit['p'].describe())         #  DataFameの関数describeの使用

with open('param_p.pkl', 'wb') as f:
    pickle.dump(df_fit['p'], f)       #  パラメータpのMCMCサンプルの保存
