import numpy as np
import stan
import pickle

x = [54, 73, 47, 61, 62, 53, 58, 63, 65, 58,
     68, 69, 49, 63, 62, 60, 63, 64, 44, 61]

with open('x.pkl', 'wb') as f:
    pickle.dump(x, f)

StanCode = """
    data {
        int n;
        array[n] real x;
        real min_x;
        real max_x;
    }
    parameters {
        real mu;
        real<lower=0> sgm;
    }
    model {
        mu ~ normal((min_x+max_x)/2, max_x-min_x);    //  muの事前分布
        sgm ~ exponential(1/(max_x-min_x));           //  sgmの事前分布
        x ~ normal(mu, sgm);                          //  正規分布モデル
    }
"""

sm = stan.build(StanCode, data={'n':len(x), 'x':x,
                                'min_x':np.min(x), 'max_x':np.max(x)})
fit = sm.sample()

df_fit = fit.to_frame()
mus = df_fit['mu']          #  muの事後分布サンプル
sgms = df_fit['sgm']        #  sgmの事後分布サンプル
print(mus.describe())
print(sgms.describe())

with open('mus.pkl', 'wb') as f:
    pickle.dump(mus, f)
    
with open('sgms.pkl', 'wb') as f:
    pickle.dump(sgms, f)
